@@ -9,7 +9,7 @@ import uuid
 
 
 class Client:
-    """ A Labelboost Client, containing a Labelbox Client, that can perform a plethora of helper functions
+    """ A Labelbase Client, containing a Labelbox Client, that can perform a plethora of helper functions
     Args:
         lb_api_key                  :   Required (str) - Labelbox API Key
         lb_endpoint                 :   Optinoal (bool) - Labelbox GraphQL endpoint
@@ -21,33 +21,6 @@ class Client:
     """
     def __init__(self, lb_api_key=None, lb_endpoint='https://api.labelbox.com/graphql', lb_enable_experimental=False, lb_app_url="https://app.labelbox.com"):  
         self.lb_client = labelboxClient(lb_api_key, endpoint=lb_endpoint, enable_experimental=lb_enable_experimental, app_url=lb_app_url)
-    
-    def refresh_metadata_ontology(self):
-        """ Refreshes a Labelbox Metadata Ontology
-        Returns 
-            lb_mdo              :   labelbox.schema.data_row_metadata.DataRowMetadataOntology
-            lb_metadata_names   :   List of metadata field names from a Labelbox metadata ontology
-        """
-        lb_mdo = self.lb_client.get_data_row_metadata_ontology()
-        lb_metadata_names = [field['name'] for field in lb_mdo._get_ontology()]
-        return lb_mdo, lb_metadata_names
-    
-    def enforce_metadata_index(self, metadata_index:dict, verbose:bool=False):
-        """ Ensure your metadata_index is in the proper format. Returns True if it is, and False if it is not
-        Args:
-            metadata_index      :   Required (dict) - Dictionary where {key=metadata_field_name : value=metadata_type}
-            verbose             :   Required (bool) - If True, prints information about code execution
-        Returns:
-            True if the metadata_index is valid, False if not
-        """
-        for metadata_field_name in metadata_index:
-            if metadata_index[metadata_field_name] not in ["enum", "string", "datetime", "number"]:
-                if verbose:
-                    print(f"Invalid value in metadata_index for key {metadata_field_name} - must be `enum`, `string`, `datetime`, or `number`")
-                return False
-        if verbose:
-            print(f"Valid metadata_index")
-        return True
       
     def sync_metadata_fields(self, table, get_columns_function, add_column_function, get_unique_values_function, metadata_index:dict={}, verbose:bool=False):
         """ Ensures Labelbox's Metadata Ontology and your input have all necessary metadata fields / columns given a metadata_index
@@ -62,13 +35,11 @@ class Client:
             Updated table if successful, False if not
         """
         # Get your metadata ontology, grab all the metadata field names
-        lb_mdo, lb_metadata_names = self.refresh_metadata_ontology()
+        lb_mdo, lb_metadata_names = connector.refresh_metadata_ontology(self.lb_client)
         # Convert your meatdata_index values from strings into labelbox.schema.data_row_metadata.DataRowMetadataKind types
         conversion = {"enum" : DataRowMetadataKind.enum, "string" : DataRowMetadataKind.string, "datetime" : DataRowMetadataKind.datetime, "number" : DataRowMetadataKind.number}
         # Check to make sure the value in your metadata index is one of the accepted values        
-        check = self.enforce_metadata_index(metadata_index, verbose)
-        if not check:
-            return False
+        connector.enforce_metadata_index(metadata_index, verbose)
         # If your table doesn't have columns for all your metadata_field_names, make columns for them
         if metadata_index:
             column_names = get_columns_function(table)
@@ -82,9 +53,9 @@ class Client:
             if metadata_field_name not in lb_metadata_names:
                 enum_options = get_unique_values_function(table, metadata_field_name) if metadata_type == "enum" else []
                 lb_mdo.create_schema(name=metadata_field_name, kind=conversion[metadata_type], options=enum_options)
-                lb_mdo, lb_metadata_names = self.refresh_metadata_ontology()
+                lb_mdo, lb_metadata_names = connector.refresh_metadata_ontology(self.lb_client)
         if 'lb_integration_source' not in lb_metadata_names:
-            lb_mdo.create_schema(name='lb_integration_source', kind=conversion["string"])      
+            lb_mdo.create_schema(name='lb_integration_source', kind=conversion["string"])
         return table      
 
     def get_metadata_schema_to_name_key(self, lb_mdo=False, divider="///", invert=False):
@@ -143,16 +114,19 @@ class Client:
                     if verbose:
                         print(f"Warning: Global keys in this upload are in use by deleted data rows, clearing all global keys from deleted data rows")
                     self.lb_client.clear_global_keys(payload['deletedDataRowGlobalKeys'])
+                    payload = connector.check_global_keys(self.lb_client, global_keys_list)
+                    continue
                 # If global keys are taken by existing data rows, either skip them on upload or update the global key to have a "_{loop_counter}" suffix
-                elif payload['fetchedDataRows']:
+                if payload['fetchedDataRows']:
                     loop_counter += 1                    
                     if verbose and skip_duplicates:
                         print(f"Warning: Global keys in this upload are in use by active data rows, skipping the upload of data rows affected")         
                     elif verbose:
-                        print(f"Warning: Global keys in this upload are in use by active data rows, adding the following suffix to affected data rows: '{divider}{loop_counter}'")   
+                        print(f"Warning: Global keys in this upload are in use by active data rows, attempting to add the following suffix to affected data rows: '{divider}{loop_counter}'")   
                     for i in range(0, len(payload['fetchedDataRows'])):
                         current_global_key = str(global_keys_list[i])
-                        new_global_key = f"{current_global_key[:-3]}{divider}{loop_counter}" if current_global_key[-3:-1] == divider else f"{current_global_key}{divider}{loop_counter}"
+                        # Add (or replace) a suffix to your working global_key
+                        new_global_key = f"{current_global_key[:-len(divider)]}{divider}{loop_counter}" if current_global_key[-len(divider):-1] == divider else f"{current_global_key}{divider}{loop_counter}"
                         if payload['fetchedDataRows'][i] != "":                            
                             if skip_duplicates:
                                 del global_key_to_upload_dict[current_global_key] # Delete this data_row_upload_dict from your upload_dict
@@ -160,9 +134,9 @@ class Client:
                                 new_upload_dict = global_key_to_upload_dict[current_global_key] # Grab the existing data_row_upload_dict
                                 del global_key_to_upload_dict[current_global_key] # Delete this data_row_upload_dict from your upload_dict
                                 new_upload_dict['global_key'] = new_global_key # Put new global key values in this data_row_upload_dict
-                                global_key_to_upload_dict[new_global_key] = new_upload_dict # Add your new data_row_upload_dict to your upload_dict
+                                global_key_to_upload_dict[new_global_key] = new_upload_dict # Add your new data_row_upload_dict to your upload_dict                               
                     global_keys_list = list(global_key_to_upload_dict.keys())
-                payload = connector.check_global_keys(self.lb_client, global_keys_list)
+                    payload = connector.check_global_keys(self.lb_client, global_keys_list) 
         upload_list = list(global_key_to_upload_dict.values())
         if verbose:
             print(f'Beginning data row upload: uploading {len(upload_list)} data rows')
@@ -308,7 +282,7 @@ class Client:
                     print(f'Success: upload batch number {batch_number} complete')               
         return []
 
-     ## Update for datetime later
+    ## Update for datetime later
     def process_metadata_value(self, metadata_value, metadata_type:str, parent_name:str, metadata_name_key_to_schema:dict, divider:str="///"):
         """ Processes inbound values to ensure only valid values are added as metadata to Labelbox given the metadata type. Returns None if invalid or None
         Args:
